import{s as a,c as s,a as e,C as t}from"./chunk-B4BG7PRW-Dio3cQnp.js";import{_ as i}from"./mermaid.core-B2Qud1y0.js";import"./chunk-FMBD7UC4-SHGnrrVt.js";import"./chunk-55IACEB6-C9D_7ckb.js";import"./chunk-QN33PNHL-CV-5eix4.js";import"./index-D29nOXqd.js";var u={parser:e,get db(){return new t},renderer:s,styles:a,init:i(r=>{r.class||(r.class={}),r.class.arrowMarkerAbsolute=r.arrowMarkerAbsolute},"init")};export{u as diagram};
