import{s as e,b as r,a,S as s}from"./chunk-DI55MBZ5-Cr3NhrWm.js";import{_ as i}from"./mermaid.core-B2Qud1y0.js";import"./chunk-55IACEB6-C9D_7ckb.js";import"./chunk-QN33PNHL-CV-5eix4.js";import"./index-D29nOXqd.js";var p={parser:a,get db(){return new s(2)},renderer:r,styles:e,init:i(t=>{t.state||(t.state={}),t.state.arrowMarkerAbsolute=t.arrowMarkerAbsolute},"init")};export{p as diagram};
